import { Inject, Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { BaseService, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { Order } from '../../entity/dining/order.entity';
import { ConfigService } from '../system/config.service';
import * as dayjs from 'dayjs';
import * as isSameOrBefore from 'dayjs/plugin/isSameOrBefore';
import * as isBetween from 'dayjs/plugin/isBetween';
import { pick, get } from 'lodash';
dayjs.extend(isSameOrBefore);
dayjs.extend(isBetween);
type modelType = ReturnModelType<typeof Order>;
@Provide()
export class OrderService extends BaseService<modelType> {
  @InjectEntityModel(Order)
  model: modelType;

  @Inject()
  configService: ConfigService;

  async computedMeal(date: string) {
    const mealList = await this.model.find({ date }).lean();
    // 将早餐,午餐,晚餐组成数据格式如下:
    /**
     *[
     * { meal: 早餐, m_id: 菜品1数量, m_id 菜品2数量, .... },
     * { meal: 午餐, m_id: 菜品1数量, m_id 菜品2数量, .... },
     * { meal: 晚餐, m_id: 菜品1数量, m_id 菜品2数量, .... }
     * ]
     * */
    const allBreakFastList = [];
    const allLunchList = [];
    const allDinnerList = [];
    const menuAllList = [];
    const merge = (data, type) => {
      let list;
      if (type === 'breakfast') list = allBreakFastList;
      else if (type === 'lunch') list = allLunchList;
      else if (type === 'dinner') list = allDinnerList;
      else return;
      for (const item of data) {
        const m = pick(item, ['_id', 'name', 'num']);
        if (!('_id' in m)) return;
        // 菜品数量按早午晚合并
        const am = list.find(f => f._id === m._id);
        if (!am) list.push(m);
        else {
          am.num = am.num + m.num;
        }
        // 菜品合并
        const i = menuAllList.findIndex(f => f._id === m._id);
        if (i <= -1) menuAllList.push(pick(item, ['_id', 'name']));
      }
    };
    for (const dm of mealList) {
      merge(get(dm, 'breakfast.list', []), 'breakfast');
      merge(get(dm, 'lunch.list', []), 'lunch');
      merge(get(dm, 'dinner.list', []), 'dinner');
    }
    const assort = (list, type) => {
      const obj: any = {};
      let key = type.split('')[0];
      if (type === 'breakfast') {
        key = 'b';
        obj.meal = '早餐';
      } else if (type === 'lunch') {
        key = 'l';
        obj.meal = '午餐';
      } else if (type === 'dinner') {
        key = 'd';
        obj.meal = '晚餐';
      }
      for (const i of list) {
        const { _id, num } = i;
        obj[`${key}_${_id}`] = num;
        obj.key = key;
      }
      return obj;
    };
    const bobj = assort(allBreakFastList, 'breakfast');
    const lobj = assort(allLunchList, 'lunch');
    const dobj = assort(allDinnerList, 'dinner');
    const data = [bobj, lobj, dobj];
    return { menuAllList, data };
  }

  // 根据餐id 获取某餐
  async getMeal(id: string) {
    const query = {
      $or: [{ 'breakfast._id': id }, { 'lunch._id': id }, { 'dinner._id': id }],
    };
    const order = await this.model.findOne(query);
    if (!order) return;
    let meal;
    if (id === get(order, 'breakfast._id')) meal = get(order, 'breakfast');
    else if (id === get(order, 'lunch._id')) meal = get(order, 'lunch');
    else if (id === get(order, 'dinner._id')) meal = get(order, 'dinner');
    return meal;
  }

  // 根据餐id, 取某餐
  async useMeal(id: string) {
    const query = {
      $or: [{ 'breakfast._id': id }, { 'lunch._id': id }, { 'dinner._id': id }],
    };
    const order = await this.model.findOne(query);
    if (!order) throw new ServiceError('未找到该餐的数据!', FrameworkErrorEnum.DATA_NOT_FOUND);
    let meal;
    if (id === get(order, 'breakfast._id')) meal = get(order, 'breakfast');
    else if (id === get(order, 'lunch._id')) meal = get(order, 'lunch');
    else if (id === get(order, 'dinner._id')) meal = get(order, 'dinner');
    meal.is_use = true;
    await order.save();
  }

  async mealCard(openid: string) {
    // 找到三餐时间设置,并判断当前时间处于那顿
    const config = await this.configService.getConfig();
    if (!config) throw new ServiceError('未找到三餐时间设置!', FrameworkErrorEnum.BAD_ARGS);
    const { b_time, l_time, d_time } = config;
    const date = dayjs().format('YYYY-MM-DD');
    const query = { openid, date };
    const fields: any = { date: 1 };
    /**
     * 三餐时间默认生成.按无法删除方式取值处理即可
     * 显示情况:
     *  1.早饭开始时间前: 显示早饭内容
     *  2.早饭结束时间前: 显示早饭内容
     *  3.早饭结束时间后-午饭开始时间前: 显示午饭内容
     *  4.午饭结束时间前: 显示午饭内容
     *  5.午饭结束时间后-晚饭开始时间前: 显示晚饭内容
     *  6.晚饭结束时间前: 显示晚饭内容
     *  7.晚饭结束时间后: 显示明天早饭
     * */
    /**早餐开始时间 */
    // const b_s = `${date} ${b_time[0]}`;
    /**早餐结束时间 */
    const b_e = `${date} ${b_time[1]}`;
    /**午餐开始时间 */
    const l_s = `${date} ${l_time[0]}`;
    /**午餐结束时间 */
    const l_e = `${date} ${l_time[1]}`;
    /**晚餐开始时间 */
    const d_s = `${date} ${d_time[0]}`;
    /**晚餐结束时间 */
    const d_e = `${date} ${d_time[1]}`;
    if (dayjs().isSameOrBefore(b_e)) {
      // 1&2
      fields.breakfast = 1;
    } else if (dayjs().isBetween(b_e, l_s) && dayjs().isSameOrBefore(l_e)) {
      // 3+4
      fields.lunch = 1;
    } else if (dayjs().isBetween(l_e, d_s) && dayjs().isSameOrBefore(d_e)) {
      // 5+6
      fields.dinner = 1;
    } else if (dayjs().isAfter(d_e)) {
      query.date = dayjs().add(1, 'day').format('YYYY-MM-DD');
      fields.breakfast = 1;
    }
    const data = await this.model.findOne(query, fields).lean();
    return data;
  }
}
